// NuclearReactorMonitor.java

class TemperatureException extends Exception
{
   private double temperature;

   TemperatureException ()
   {
      super ("Control is broken!");
   }

   TemperatureException (String s)
   {
      super (s);
   }

   public void setTemperature (double temperature)
   {
      this.temperature = temperature;
   }

   public double getTemperature ()
   {
      return temperature;
   }
}

class NuclearReactorMonitor
{
   static double coreTemperature = 10000.0;

   public static void main (String [] args)
   {
      try
      {
          monitorReactorCore ();
          return;
      }
      catch (TemperatureException e)
      {
          System.out.println (e.getMessage ());
          System.out.println ("Core temperature has reached: "
                              + e.getTemperature ());
      }
      finally
      {
          System.out.println ("Shutting down reactor!");
      }
   }

   static int applyControl ()
   {
      return (int) (Math.random () * 3);
   }

   static double fluctuation ()
   {
      double amount;

      amount = Math.random () * 100;

      if (Math.random () < 0.5)
          amount = -amount;

      return amount;
   }

   static void monitorReactorCore () throws TemperatureException
   {
      while (true)
      {
         coreTemperature += fluctuation ();

         if (coreTemperature > 15000.0)
         {
             switch (applyControl ())
             {
                case 0: coreTemperature = 10000.0;
                        break;

                case 1: TemperatureException te =
                            new TemperatureException ("Control is jammed!");
                        te.setTemperature (coreTemperature);
                        throw te;

                case 2: TemperatureException te2 =
                            new TemperatureException ();
                        te2.setTemperature (coreTemperature);
                        throw te2;
             }
         }

         // If core temperature drops below 9000 degrees ...
         // ... reactor is cooling down and will stop nuclear ...
         // ... reaction.

         if (coreTemperature < 9000.0)
             break;
      } 
   }
}
