// BSTDemo.java

import java.io.*;

class treeNode
{
   int data;         // Data being stored.
   treeNode left;    // Left subtree reference.
   treeNode right;  // Right subtree reference.

   public treeNode (int data)
   {
      this.data = data;
      left = right = null;
   }

   public void insert (int data)
   {
      if (this.data > data) // data argument precedes current data
      {   
          // if left-most leaf node reached then insert new node
          // as its left-most leaf node.  Otherwise, keep
          // searching left.

          if (left == null)
              left = new treeNode (data);
          else
              left.insert (data);
      }
      else
      if (this.data < data) // data argument follows current data   
      {
          // if right-most leaf node reached then insert new node
          // as its right-most leaf node.  Otherwise, keep
          // searching right.

          if (right == null)
              right = new treeNode (data);
          else
              right.insert (data);
      }
   }
}

class BSTDemo
{
   public static void main (String [] args)
   {
      int [] numbers = { 79, 7, 33, 61, 25, 57, 46, 62, 15 };

      treeNode root = new treeNode (numbers [0]);

      for (int i = 1; i < numbers.length; i++)
           root.insert (numbers [i]);

      System.out.println ("In-order traversal");
      inOrder (root);
      System.out.println ("\n");

      System.out.println ("Pre-order traversal");
      preOrder (root);
      System.out.println ("\n");

      System.out.println ("Post-order traversal");
      postOrder (root);
      System.out.println ("");
   }

   static void inOrder (treeNode root)
   {
      // If either the root node or the current node is null,
      // signifying that a leaf node has been reached, return.

      if (root == null)
          return;

      // Display all left-most nodes (i.e., nodes whose integers
      // precede integers in the current node).

      inOrder (root.left);

      // Display the current node's integer data.

      System.out.print (root.data + " ");

      // Display all right-most nodes (i.e., nodes whose integers
      // follow integer in the current node).

      inOrder (root.right);
   }

   static void preOrder (treeNode root)
   {
      // If either the root node or the current node is null,
      // signifying that a leaf node has been reached, return.

      if (root == null)
          return;

      // Display the current node's integer data.

      System.out.print (root.data + " ");

      // Display all left-most nodes (i.e., nodes whose integers
      // precede integers in the current node).

      preOrder (root.left);

      // Display all right-most nodes (i.e., nodes whose integers
      // follow integer in the current node).

      preOrder (root.right);
   }

   static void postOrder (treeNode root)
   {
      // If either the root node or the current node is null,
      // signifying that a leaf node has been reached, return.

      if (root == null)
          return;

      // Display all left-most nodes (i.e., nodes whose integers
      // precede integers in the current node).

      postOrder (root.left);

      // Display all right-most nodes (i.e., nodes whose integers
      // follow integer in the current node).

      postOrder (root.right);

      // Display the current node's integer data.

      System.out.print (root.data + " ");
   }
}
